#include <Wire.h> // ultrasonic sensor
#include <LiquidCrystal_I2C.h>  // display
#include <TinyStepper_28BYJ_48.h> // stepper motor

int inside = 0;

int lastStateSensor1 = 0;
int lastStateSensor2 = 0;

const byte trigPin1 = 2;
const byte echoPin1 = 3;

const byte trigPin2 = 4;
const byte echoPin2 = 5;

const byte motorPin1 = 8; 
const byte motorPin2 = 9;
const byte motorPin3 = 10;
const byte motorPin4 = 11;

const int STEPS_PER_REVOLUTION = 2038;
TinyStepper_28BYJ_48 stepper;

LiquidCrystal_I2C lcd(0x27, 16, 2); // Schimba adresa I2C daca este necesar

long time1, time2;

int barrierState = 0; // 0 -> inchisa , 1 -> deschisa

void setup() {
  // put your setup code here, to run once:
  Wire.begin();
  pinMode(trigPin1, OUTPUT);
  pinMode(echoPin1, INPUT);

  pinMode(trigPin2, OUTPUT);
  pinMode(echoPin2, INPUT);

  //Serial.begin(9600);
  lcd.init();
  lcd.backlight();

  stepper.connectToPins(motorPin1, motorPin2, motorPin3, motorPin4);
  stepper.setSpeedInStepsPerSecond(256);
  stepper.setAccelerationInStepsPerSecondPerSecond(512);
  //stepper.moveRelativeInSteps(-200);
}

long readSensor1() {
  digitalWrite(trigPin1, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin1, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin1, LOW);
  long time = pulseIn(echoPin1, HIGH);
  return time;
}

long readSensor2() {
  digitalWrite(trigPin2, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin2, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin2, LOW);
  long time = pulseIn(echoPin2, HIGH);
  return time;
}

void openBarrier() {
  stepper.moveRelativeInSteps(-420);
}

void closeBarrier() {
  stepper.moveRelativeInSteps(420);
}

void loop() {
  // put your main code here, to run repeatedly:
  long time1 = readSensor1(); long distance1 = time1 / 58;
  delay(100);
  long time2 = readSensor2(); long distance2 = time2 / 58;
 
  if(distance1 > 30 && distance2 > 30){ // nicio masina nu este in "raza de actiune" a vreunui senzor
    if(barrierState == 1){
      barrierState = 0;
      closeBarrier();
    }
  } else {
    // daca o masina este in raza de actiune a unui senzor
    // ridicam bariera daca nu este deja ridicata si permitem unei
    // masini sa intre doar daca parcarea nu este plina
    if(barrierState == 0){ 
      if(distance2 <= 30 || inside < 5){
        barrierState = 1;
        openBarrier();
      }
      
    }
  }

  if(distance2 <= 30 && distance1 > 30 && inside < 5){
    // daca senzorul din interior detecteaza o masina, iar senzorul de la 
    // intrare in parcare a fost anterior aprins si parcarea nu este plina
    // inseamna ca a intrat o masina in parcare si incrementam numarul de masini
    // din paracare
    if(lastStateSensor1 == 1){
      lastStateSensor1 = 0;
      inside++;
    }
  }
  if(distance1 <= 30 && distance2 > 30 && inside > 0){
    // analog ca anterior decrementam numarul de masini din parcare
    // daca o masina a iesit din parcare
    if(lastStateSensor2 == 1){
      lastStateSensor2 = 0;
      inside--;
    }
  }

  // updatam de fiecare data variabilele ce retin
  // ultima stare inainte de cea curenta a senzorilor
  if(distance1 > 30){
    lastStateSensor1 = 0;
  } else {
    lastStateSensor1 = 1;
  }

  if(distance2 > 30){
    lastStateSensor2 = 0;
  } else {
    lastStateSensor2 = 1;
  }


  lcd.clear();
  lcd.setCursor(0, 0);
  if(inside < 5){
    lcd.print("Available: " + String(5 - inside));
  }
  else {
    lcd.print("No spaces availabe !");
  }
  
  lcd.setCursor(0, 1);
  lcd.print(distance1);
  lcd.print(" ");
  lcd.print(distance2);

}
